/* FILE: hexanisotropy.h            -*-Mode: c++-*-
 *
 * Hex Anisotropy, derived from Oxs_Energy class.
 */

#ifndef _OXS_HEXANISOTROPY
#define _OXS_HEXANISOTROPY

#include "nb.h"
#include "threevector.h"
#include "energy.h"
#include "key.h"
#include "simstate.h"
#include "mesh.h"
#include "meshvalue.h"
#include "scalarfield.h"
#include "vectorfield.h"

/* End includes */

// For hexagonal symmetry, the anisotropy energy can be written as
//
//      K_1 \sin^2(\theta)
//   +  K_2 \sin^4(\theta)
//   +  K_3 \sin^6(\theta)
//   +  K_4 \sin^6(\theta) \cos(6\phi)
//
// where theta is angle between m and c-axis (axis1), and
// phi is angle between m and basal-plane axis (axis 2).

class Oxs_HexAnisotropy:public Oxs_Energy {
private:
  Oxs_OwnedPointer<Oxs_ScalarField> K1_init;
  Oxs_OwnedPointer<Oxs_ScalarField> K2_init;
  Oxs_OwnedPointer<Oxs_ScalarField> K3_init;
  Oxs_OwnedPointer<Oxs_ScalarField> K4_init;
  Oxs_OwnedPointer<Oxs_VectorField> axis1_init;
  Oxs_OwnedPointer<Oxs_VectorField> axis2_init;
  mutable UINT4m mesh_id;
  mutable Oxs_MeshValue<REAL8m> K1;
  mutable Oxs_MeshValue<REAL8m> K2;
  mutable Oxs_MeshValue<REAL8m> K3;
  mutable Oxs_MeshValue<REAL8m> K4;
  mutable Oxs_MeshValue<ThreeVector> axis1;
  mutable Oxs_MeshValue<ThreeVector> axis2;
  /// K#, axis1 and axis2 are cached values filled by corresponding
  /// *_init members when a change in mesh is detected.

protected:
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const;

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_HexAnisotropy(const char* name,  // Child instance id
		      Oxs_Director* newdtr, // App director
		      const char* argstr);  // MIF input block parameters

  virtual ~Oxs_HexAnisotropy() {}
  virtual BOOL Init();
};


#endif // _OXS_HEXANISOTROPY
