# FILE: sgi.tcl
#
# Configuration feature definitions for the configuration 'sgi'
#
# Editing instructions begin at "START EDIT HERE" below.

set config [Oc_Config RunPlatform]

if {![string match [string tolower [file rootname [file tail [info script]]]] \
        [$config GetValue platform_name]]} {
    error "Configuration cache file '[info script]' 
sourced by '[$config GetValue platform_name]'"
}

########################################################################
# START EDIT HERE
# In order to properly build, install, and run on your computing 
# platform, the OOMMF software must know certain features of your
# computing environment.  In this file are lines which set the value of 
# certain features of your computing environment.  Each line looks like:
#
# $config SetValue <feature> {<value>}
#
# where each <feature> is the name of some feature of interest,
# and <value> is the value which is assigned to that feature in a
# description of your computing environment.  Your task is to edit
# the values as necessary to properly describe your computing 
# environment.
#
# The character '#' at the beginning of a line is a comment character.
# It causes the contents of that line to be ignored.  To select
# among lines providing alternative values for a feature, uncomment the
# line containing the proper value.
#
# The features in this file are divided into three sections.  The first
# section (REQUIRED CONFIGURATION) includes features which require you 
# to provide a value.  The second section (OPTIONAL CONFIGURATION)
# includes features which have usable default values, but which you
# may wish to customize.  The third section (ADVANCED CONFIGURATION)
# contains features which you probably do not need or want to change
# without a good reason.
########################################################################
# REQUIRED CONFIGURATION

# Set the feature 'program_compiler_c++' to the program to run on this 
# platform to compile source code files written in the language C++ into 
# object files.  Select from the choices below.  If the compiler is not 
# in your path, be sure to use the whole pathname.  Also include any 
# options required to instruct your compiler to only compile, not link.  
#
# If your compiler is not listed below, additional features will
# have to be added in the ADVANCED CONFIGURATION section below to 
# describe to the OOMMF software how to operate your compiler.  Send
# e-mail to the OOMMF developers for assistance.
#
# SGI's MIPSpro C++ compiler
# <URL:http://www.sgi.com/developers/devtools/languages/mipspro.html>
# Specify -32, -o32, -n32 or -64 as necessary to match your Tcl/Tk
#   libraries.  Likely matches are -n32 for Tcl/Tk 8.0.3 and newer,
#   and -o32 for older Tcl/Tk.  If you are using the -64 switch,
#   make sure too that the system 64-bit library directories (e.g.,
#   /usr/lib64) are on LD_LIBRARY_PATH.  Also, you will have to
#   have a 64-bit build of Tcl/Tk.
$config SetValue program_compiler_c++ {CC -n32 -LANG:std:libc_in_namespace_std=OFF -c}
#$config SetValue program_compiler_c++ {CC -64 -LANG:std:libc_in_namespace_std=OFF -c}
# The "libc_in_namespace_std=OFF" option is protection against broken
# header files in some versions of the compiler and/or broken -Ofast
# switches. YMMV.
#
# The GNU C++ compiler 'g++'
# <URL:http://www.gnu.org/software/gcc/gcc.html>
# <URL:http://egcs.cygnus.com/>
# It appears that gcc will not build -o32 code.  So must link
#   against -n32 Tcl/Tk libraries, which is the default for
#   Tcl/Tk 8.0.3 and later.
set gnucc "g++" ;# OOMMF 1.1 can use either gcc or g++; 1.2+ needs g++
#$config SetValue program_compiler_c++ "$gnucc -c"

########################################################################
# OPTIONAL CONFIGURATION

# Set the feature 'path_directory_temporary' to the name of an existing 
# directory on your computer in which OOMMF software should write 
# temporary files.  All OOMMF users must have write access to this 
# directory.
#
# $config SetValue path_directory_temporary {/tmp}

########################################################################
# ADVANCED CONFIGURATION

# Compile in thread support?
set oommf_threads [$config GetValue oommf_threads]

# Compiler option processing...
set ccbasename [file tail [lindex [$config GetValue program_compiler_c++] 0]]
set id32 ""
set id64 ""
set langopt ""
foreach _ [lrange [$config GetValue program_compiler_c++] 1 end] {
    if {[string match -*32 $_]} {
        set id32 $_
    }
    if {[string match -*64 $_]} {
        set id64 $_
    }
    if {[string match -LANG* $_]} {
	set langopt $_
    }
}
unset _
if {![info exists gnucc]} {
	set gnucc {}  ;# Safety.
}
if {[string match CC $ccbasename]} {
    # ...for SGI's compiler CC
    set opts {}
    #lappend opts -O-OPT:Olimit=25000
    lappend opts -Ofast
    #lappend opts -O0
    ## If you specify -Ofast here, then you must also specify -IPA
    ## in the program_linker line below.  Also, at least some versions
    ## of CC don't like the combination '-32 -Ofast', so you may have
    ## to avoid 'CC -32 -c' above in favor of 'CC -n32 -c'.

    # Compile in thread support?
    if {$oommf_threads} {
       lappend opts -DTCL_THREADS -DOOMMF_THREADS=1
    } else {
       lappend opts -DOOMMF_THREADS=0
    }

    $config SetValue program_compiler_c++_option_opt "format \"$opts\""
    # NOTE: If you want good performance, be sure to edit ../options.tcl
    #  or ../local/options.tcl to include the line
    #    Oc_Option Add * Platform cflags {-def NDEBUG}
    #  so that the NDEBUG symbol is defined during compile.

    $config SetValue program_compiler_c++_option_out {format "-o \"%s\""}
    $config SetValue program_compiler_c++_option_src {format \"%s\"}
    $config SetValue program_compiler_c++_option_inc {format "\"-I%s\""}
    $config SetValue program_compiler_c++_option_warn {format "-fullwarn"}
    $config SetValue program_compiler_c++_option_debug {format "-g"}
    $config SetValue program_compiler_c++_option_def {format "\"-D%s\""}

    # Wide floating point type.  Defaults to double, but you can
    # change this to "long double" for extra precision at greatly
    # reduced speed.
    # $config SetValue program_compiler_c++_typedef_realwide "long double"

    # Experimental: The REAL8m type is intended to be at least
    # 8 bytes wide.  Generally REAL8m is typedef'ed to double,
    # but you can try setting this to "long double" for extra
    # precision (and extra slowness).
    # $config SetValue program_compiler_c++_typedef_real8m "long double"

    # Broken cmath header?
    $config SetValue program_compiler_c++_property_bad_cmath_header 1

} elseif {[string match $gnucc $ccbasename]} {
    # ...for GNU g++ C++ compiler

    set opts {}
    lappend opts "-O%s"  ;# Default optimization

    # Compile in thread support?
    if {$oommf_threads} {
       lappend opts -DTCL_THREADS -DOOMMF_THREADS=1
    } else {
       lappend opts -DOOMMF_THREADS=0
    }

    $config SetValue program_compiler_c++_option_opt "format \"$opts\""
    # NOTE: If you want good performance, be sure to edit ../options.tcl
    #  or ../local/options.tcl to include the line
    #    Oc_Option Add * Platform cflags {-def NDEBUG}
    #  so that the NDEBUG symbol is defined during compile.

    $config SetValue program_compiler_c++_option_out {format "-o \"%s\""}
    $config SetValue program_compiler_c++_option_src {format \"%s\"}
    $config SetValue program_compiler_c++_option_inc {format "\"-I%s\""}
 
    # Compiler warnings:
    # Omitted: -Wredundant-decls -W -shadow
    # I would also like to use -Wcast-qual, but casting away const is
    # needed on some occasions to provide "conceptual const" functions in
    # place of "bitwise const"; cf. p76-78 of Meyer's book, "Effective C++."
    #
    # NOTE: -Wno-uninitialized is required after -Wall by gcc 2.8+ because
    # of an apparent bug
    $config SetValue program_compiler_c++_option_warn {format "-Wall \
	-Wpointer-arith -Wbad-function-cast -Wwrite-strings \
	-Wstrict-prototypes -Wmissing-declarations \
	-Wnested-externs -Winline -Woverloaded-virtual -Wsynth -Werror \
	-Wcast-align"}
    $config SetValue program_compiler_c++_option_debug {format "-g"}
    $config SetValue program_compiler_c++_option_def {format "\"-D%s\""}

    # Wide floating point type.  Defaults to double, but you can
    # change this to "long double" for extra precision at greatly
    # reduced speed.
    # $config SetValue program_compiler_c++_typedef_realwide "long double"

    # Experimental: The REAL8m type is intended to be at least
    # 8 bytes wide.  Generally REAL8m is typedef'ed to double,
    # but you can try setting this to "long double" for extra
    # precision (and extra slowness).
    # $config SetValue program_compiler_c++_typedef_real8m "long double"

    # Directories to exclude from explicit include search path, i.e.,
    # the -I list.  Some versions of gcc complain if "system" directories
    # appear in the -I list.
    $config SetValue \
    	program_compiler_c++_system_include_path [list /usr/include]
}

# The program to run on this platform to link together object files and
# library files to create an executable binary.
#
# Use the selected compiler to control the linking.
if {[string match CC [file tail [lindex \
        [$config GetValue program_compiler_c++] 0]]]} {
    set linkcmdline [lindex [$config GetValue program_compiler_c++] 0]
    if {![string match {} $id32]} {lappend linkcmdline $id32}
    if {![string match {} $id64]} {lappend linkcmdline $id64}
    if {![string match {} $langopt]} {lappend linkcmdline $langopt}

    # Disable "Multiply defined" warnings from linker
    lappend linkcmdline "-Wl,-woff15"

    # Include -IPA if you specified -Ofast in the compile line above.
    # There is an optimization bug affecting routines taking a variable
    # number of args (C++ "..." specification; cf. stdargs.h); one fix
    # is to turn off "aggressive interprocedural constant propagation"
    # with the linker option '-IPA:aggr_cprop=OFF'.
    lappend linkcmdline "-IPA:aggr_cprop=OFF"

    $config SetValue program_linker $linkcmdline
    unset linkcmdline
} else {
    $config SetValue program_linker [lindex \
            [$config GetValue program_compiler_c++] 0]
}

# Linker option processing...
set lbasename [file tail [lindex [$config GetValue program_linker] 0]]
if {[string match CC $lbasename]} {
    # ...for SGI's CC as linker
    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "-o \"%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_uses_-L-l {1}
} elseif {[string match $gnucc $lbasename]} {
    # ...for GNU g++ as linker
    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "-o \"%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_uses_-L-l {1}
}
unset lbasename

# The program to run on this platform to create a single library file out
# of many object files.
if {[string match CC $ccbasename]} {
    # Use CC to get pre-linker run.
    set libmaker $ccbasename
    if {![string match {} $id32]} {lappend libmaker $id32}
    if {![string match {} $id64]} {lappend libmaker $id64}
    if {![string match {} $langopt]} {lappend libmaker $langopt}
    lappend libmaker -ar -o
    $config SetValue program_libmaker $libmaker
    unset libmaker
    # Option processing for ar
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format \"%s\"}
} elseif {[string match $gnucc $ccbasename]} {
    $config SetValue program_libmaker {ar cr}
    # Option processing for ar
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format \"%s\"}
}
unset ccbasename id32 id64 langopt gnucc


# The absolute, native filename of the null device
$config SetValue path_device_null {/dev/null}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an executable on this platform
$config SetValue script_filename_executable {format %s}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# an object file on this platform
$config SetValue script_filename_object {format %s.o}

# A partial Tcl command (or script) which when completed by lappending
# a file name stem and evaluated returns the corresponding file name for
# a static library on this platform
$config SetValue script_filename_static_library {format lib%s.a}

# A list of partial Tcl commands (or scripts) which when completed by
# lappending a file name stem and evaluated returns the corresponding
# file name for an intermediate file produced by the linker on this platform
$config SetValue script_filename_intermediate [list {format ii_files/%s.ii}]

########################################################################
unset config

