/* FILE: nb.cc                   -*-Mode: c++-*-
 *
 *	The Nuts & Bolts extension.
 *
 *	This extension provides a collection of C++ classes and functions
 * of general utility to all OOMMF applications.
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 2008-05-20 23:14:11 $
 * Last modified by: $Author: donahue $
 */

/* Header file for this extension */
#include "oc.h"
#include "evoc.h"
#include "nb.h"

/* End includes */     

int 
Nb_Init(Tcl_Interp *interp)
{
#define RETURN_TCL_ERROR                                               \
    Tcl_AddErrorInfo(interp, OC_CONST84_CHAR("\n    (in Nb_Init())")); \
    return TCL_ERROR

  if (Tcl_PkgPresent(interp, OC_CONST84_CHAR("Oc"),
		     OC_CONST84_CHAR("1.2.0.4"), 0) == NULL) {
    Tcl_AppendResult(interp,
	     OC_CONST84_CHAR("\n\t(Nb "NB_VERSION" needs Oc 1.2.0.4)"), NULL);
    RETURN_TCL_ERROR;
  }

  // Register C++-based Tcl commands of this extension
  Oc_RegisterCommand(interp,"Nb_ComputeCRCBuffer",
		     (Tcl_CmdProc *)NbComputeCRCBufferCmd);
  Oc_RegisterCommand(interp,"Nb_ComputeCRCChannel",
		     (Tcl_CmdProc *)NbComputeCRCChannelCmd);
  Oc_RegisterCommand(interp,"Nb_Gcd",(Tcl_CmdProc *)NbGcdCmd);
  Oc_RegisterCommand(interp,"Nb_RatApprox",(Tcl_CmdProc *)NbRatApprox);
  Oc_RegisterCommand(interp,"Nb_GetColor",(Tcl_CmdProc *)NbGetColor);
  Oc_RegisterCommand(interp,"Oc_Srand",(Tcl_CmdProc *)OcSrand);
  Oc_RegisterCommand(interp,"Oc_UnifRand",(Tcl_CmdProc *)OcUnifRand);
  Oc_RegisterCommand(interp,"Oc_Times",(Tcl_CmdProc *)OcTimes);
#if (SYSTEM_TYPE == UNIX)
  Oc_RegisterCommand(interp,"Nb_GetUserId",(Tcl_CmdProc *)NbGetUserId);
#endif
#if (SYSTEM_TYPE == WINDOWS)
  Oc_RegisterCommand(interp,"Nb_GetPidUserName",
                     (Tcl_CmdProc *)NbGetPidUserName);
  Oc_RegisterCommand(interp,"Nb_GetUserName",(Tcl_CmdProc *)NbGetUserName);
#endif


  // Add Tcl expr command extensions, if any.
  Oc_AddTclExprExtensions(interp);
  Oc_RegisterCommand(interp,"Oc_AddExprExtensions",
		     (Tcl_CmdProc *)OcAddTclExprExtensions);

  if (Tcl_PkgProvide(interp, OC_CONST84_CHAR("Nb"),
		     OC_CONST84_CHAR(NB_VERSION)) != TCL_OK) {
    RETURN_TCL_ERROR;
  }

  // Setup Tcl script portion of Nb extension
  if ( Oc_InitScript(interp, "Nb", NB_VERSION) != TCL_OK) {
    RETURN_TCL_ERROR;
  }

  return TCL_OK;

#undef RETURN_TCL_ERROR
}
